<?php

/**
 * @file
 * Display current visitor block.
 */

/**
 * Implements hook_block_info().
 */
function visitors_block_info() {
  $blocks = array();

  $blocks['visitors'] = array(
    'info' => t('Current visitor'),
    'cache' => DRUPAL_NO_CACHE,
  );

  return $blocks;
}

/**
 * Implements hook_block_view().
 */
function visitors_block_view($delta = '') {
  $data = array();
  $data['subject'] = t('Visitors');
  $data['content'] = theme('visitors_block_view');

  return $data;
}

/**
 * Implement a theme function for block output.
 */
function theme_visitors_block_view() {
  $list_items = array();

  visitors_block_show_total_visitors($list_items);
  visitors_block_show_unique_visitors($list_items);
  visitors_block_registered_users($list_items);
  visitors_block_last_registered_user($list_items);
  visitors_block_published_nodes($list_items);
  visitors_block_user_ip($list_items);
  visitors_block_since_date($list_items);

  return theme('item_list', array('items' => $list_items, 'type' => 'ul'));
}

/**
 * Implements hook_block_configure().
 */
function visitors_block_configure($delta = '') {

  if ($delta == 'visitors') {
    $form = array();

    $form['visitors_show_total_visitors'] = array(
      '#type' => 'checkbox',
      '#title' => t('Show Total Visitors'),
      '#default_value' => variable_get('visitors_show_total_visitors', 1),
      '#description' => t('Show Total Visitors.'),
    );

    $form['visitors_show_unique_visitor'] = array(
      '#type' => 'checkbox',
      '#title' => t('Show Unique Visitors'),
      '#default_value' => variable_get('visitors_show_unique_visitor', 1),
      '#description' => t('Show Unique Visitors based on their IP.'),
    );

    $form['visitors_registered_user'] = array(
      '#type' => 'checkbox',
      '#title' => t('Show Registered Users'),
      '#default_value' => variable_get('visitors_registered_user', 1),
      '#description' => t('Show Registered Users.'),
    );

    $form['visitors_last_registered_user'] = array(
      '#type' => 'checkbox',
      '#title' => t('Show Last Registered User'),
      '#default_value' => variable_get('visitors_last_registered_user', 1),
      '#description' => t('Show Last Registered User.'),
    );

    $form['visitors_published_nodes'] = array(
      '#type' => 'checkbox',
      '#title' => t('Show Published Nodes'),
      '#default_value' => variable_get('visitors_published_nodes', 1),
      '#description' => t('Show Published Nodes.'),
    );

    $form['visitors_user_ip'] = array(
      '#type' => 'checkbox',
      '#title' => t('Show User IP'),
      '#default_value' => variable_get('visitors_user_ip', 1),
      '#description' => t('Show User IP.'),
    );

    $form['visitors_since_date'] = array(
      '#type' => 'checkbox',
      '#title' => t('Show Since Date'),
      '#default_value' => variable_get('visitors_since_date', 1),
      '#description' => t('Show Since Date.'),
    );

    $form['visitors_start_count_total_visitors'] = array(
      '#type' => 'textfield',
      '#title' => t('Total visitors start count'),
      '#default_value' => variable_get('visitors_start_count_total_visitors', 0),
      '#description' => t('Start the count of the total visitors at this number. Useful for including the known number of visitors in the past.'),
    );

    return $form;
  }
}

/**
 * Implements hook_block_save().
 */
function visitors_block_save($delta = '', $edit = array()) {

  if ($delta == 'visitors') {
    variable_set('visitors_show_total_visitors', $edit['visitors_show_total_visitors']);
    variable_set('visitors_start_count_total_visitors', $edit['visitors_start_count_total_visitors']);
    variable_set('visitors_show_unique_visitor', $edit['visitors_show_unique_visitor']);
    variable_set('visitors_registered_user', $edit['visitors_registered_user']);
    variable_set('visitors_last_registered_user', $edit['visitors_last_registered_user']);
    variable_set('visitors_published_nodes', $edit['visitors_published_nodes']);
    variable_set('visitors_user_ip', $edit['visitors_user_ip']);
    variable_set('visitors_since_date', $edit['visitors_since_date']);
  }
}

/**
 * Display total visitors count to visitors block.
 *
 * @param array $list_items
 *   An associative array of variables, use for theme function.
 */
function visitors_block_show_total_visitors(array &$list_items) {
  if (variable_get('visitors_show_total_visitors', 1)) {
    $start_count = variable_get('visitors_start_count_total_visitors', 0);
    $query = 'SELECT COUNT(*) FROM {visitors}';
    $count = db_query($query)->fetchField() + $start_count;
    $list_items[] = sprintf('%s: %d', t('Total Visitors'), $count);
  }
}

/**
 * Display unique visitors count to visitors block.
 *
 * @param array $list_items
 *   An associative array of variables, use for theme function.
 */
function visitors_block_show_unique_visitors(array &$list_items) {
  if (variable_get('visitors_show_unique_visitor', 1)) {
    $query = 'SELECT COUNT(DISTINCT visitors_ip) FROM {visitors}';
    $unique_visitors = db_query($query)->fetchField();
    $list_items[] = sprintf('%s: %u', t('Unique Visitors'), $unique_visitors);
  }
}

/**
 * Display registered users count to visitors block.
 *
 * @param array $list_items
 *   An  associative array of variables, use for theme function.
 */
function visitors_block_registered_users(array &$list_items) {
  if (variable_get('visitors_registered_user', 1)) {
    $query = 'SELECT COUNT(*) FROM {users} WHERE uid <> 0';
    $registered_users = db_query($query)->fetchField();
    $list_items[] = sprintf(
      '%s: %u',
      t('Registered Users'),
      $registered_users
    );
  }
}

/**
 * Display last registered user to visitors block.
 *
 * @param array $list_items
 *   An associative array of variables, use for theme function.
 */
function visitors_block_last_registered_user(array &$list_items) {
  if (variable_get('visitors_last_registered_user', 1)) {
    $last_user_uid = db_select('users', 'u')
      ->fields('u', array('uid'))
      ->orderBy('uid', 'DESC')
      ->range(0, 1)
      ->execute()
      ->fetchField();
    $user = user_load($last_user_uid, TRUE);
    $list_items[] = t('Last Registered User: !last_user',
      array('!last_user' => theme('username', array('account' => $user)))
    );
  }
}

/**
 * Display published nodes count to visitors block.
 *
 * @param array $list_items
 *   An associative array of variables, use for theme function.
 */
function visitors_block_published_nodes(array &$list_items) {
  if (variable_get('visitors_published_nodes', 1)) {
    $query = 'SELECT COUNT(*) FROM {node} WHERE status = 1';
    $nodes = db_query($query)->fetchField();
    $list_items[] = sprintf('%s: %u', t('Published Nodes'), $nodes);
  }
}

/**
 * Display user ip to visitors block.
 *
 * @param array $list_items
 *   An associative array of variables, use for theme function.
 */
function visitors_block_user_ip(array &$list_items) {
  if (variable_get('visitors_user_ip', 1)) {
    $list_items[] = sprintf('%s: %s', t('Your IP'), ip_address());
  }
}

/**
 * Display the start date statistics to visitors block.
 *
 * @param array $list_items
 *   An associative array of variables, use for theme function.
 */
function visitors_block_since_date(array &$list_items) {
  if (variable_get('visitors_since_date', 1)) {
    $query = 'SELECT visitors_date_time
      FROM {visitors}
      ORDER BY visitors_date_time ASC';
    $since_date = db_query_range($query, 0, 1)->fetchField();
    $list_items[] = sprintf(
      '%s: %s',
      t('Since'),
      format_date($since_date, 'short')
    );
  }
}
